"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.provideResolver = exports.templateRoot = void 0;
const TransformPackage = __importStar(require("graphql-transformer-core"));
const amplify_cli_core_1 = require("amplify-cli-core");
const enquirer_1 = require("enquirer");
const graphql_1 = require("graphql");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const destFileMapper_1 = require("../utils/destFileMapper");
exports.templateRoot = `${__dirname}/../../resources`;
const pathToTemplateFiles = path.join(exports.templateRoot, 'lambda/resolver');
const provideResolver = async (context) => {
    const resolvers = await askGraphQLFieldsQuestions(context);
    const files = fs.readdirSync(pathToTemplateFiles);
    return Promise.resolve({
        functionTemplate: {
            sourceRoot: pathToTemplateFiles,
            sourceFiles: files,
            defaultEditorFile: path.join('src', 'index.js'),
            destMap: (0, destFileMapper_1.getDstMap)(files),
            parameters: {
                resolvers
            }
        },
    });
};
exports.provideResolver = provideResolver;
const askGraphQLFieldsQuestions = async (context) => {
    const { allResources } = await context.amplify.getResourceStatus();
    const appSyncResources = allResources.filter((resource) => resource.service === 'AppSync');
    let targetResourceName;
    if (appSyncResources.length === 0) {
        const errMessage = `
      No AppSync resources have been configured in the API category.
      Please use "amplify add api" command to create a new appsync resource`;
        context.print.error(errMessage);
        await context.usageData.emitError(new amplify_cli_core_1.ResourceDoesNotExistError(errMessage));
        (0, amplify_cli_core_1.exitOnNextTick)(0);
    }
    else if (appSyncResources.length === 1) {
        targetResourceName = appSyncResources[0].resourceName;
        context.print.success(`Selected resource ${targetResourceName}`);
    }
    else {
        const answer = await (0, enquirer_1.prompt)({
            type: 'select',
            name: 'resolverAPIResourceName',
            message: 'Choose an API resource to associate with'
        });
        targetResourceName = answer.resolverAPIResourceName;
    }
    const backendDir = context.amplify.pathManager.getBackendDirPath();
    const resourceDirPath = path.join(backendDir, 'api', targetResourceName);
    const project = await TransformPackage.readProjectConfiguration(resourceDirPath);
    const doc = (0, graphql_1.parse)(project.schema);
    const choices = [];
    for (const definition of doc.definitions) {
        if (definition.kind === graphql_1.Kind.OBJECT_TYPE_DEFINITION) {
            const typeName = definition.name;
            const fields = definition.fields || [];
            const choice = {
                name: typeName.value,
                value: typeName.value,
                choices: fields.map(field => ({
                    name: `${typeName.value}.${field.name.value}`,
                    value: `${typeName.value}.${field.name.value}`,
                    message: field.name.value
                }))
            };
            choices.push(choice);
        }
    }
    const filedsQuestions = {
        type: 'multiselect',
        name: 'fileds',
        message: 'Select fields to associate with lambda resolver',
        choices: choices
    };
    const answer = await (0, enquirer_1.prompt)(filedsQuestions);
    const resolverMap = new Map();
    answer.fileds.forEach(field => {
        var _a;
        if (!field.includes('.')) {
            return;
        }
        const [typeName, fieldName] = field.split('.');
        if (!resolverMap.has(typeName)) {
            resolverMap.set(typeName, {
                name: typeName,
                fieldResolvers: []
            });
        }
        const resolver = resolverMap.get(typeName);
        (_a = resolver === null || resolver === void 0 ? void 0 : resolver.fieldResolvers) === null || _a === void 0 ? void 0 : _a.push({
            name: fieldName,
        });
    });
    return Array.from(resolverMap.values());
};
//# sourceMappingURL=resolverProvider.js.map